
%--------------------------------------------------------------------------
%                               Variables                                 
%--------------------------------------------------------------------------
var   

% Standard Variables
c           % consumption
i           % investment
k           % capital services
kbar        % physical capital
l           % hours
lambda      % real marginal utility of nominal income
p           % inflation
q           % Tobin's q
r           % nominal interest rate
rk          % real rental rate on capital
s           % marginal cost
u           % utilization rate
w           % real wage
x           % gdp
y           % output

% Financial Variables
a           % bank assets
b           % face value of debt normalized by bank assets
B           % bank debt
Bbar        % face value of bank debt
hc          % haircut = inverse of leverage
mu          % lagrange multiplier on bank's program
n           % net worth
o           % threshold level of island-specific shock
phi         % leverage
F           % cdf standard firm
Ft          % cdf substandard firm
pi          % put option value on standard firm
pit         % put option value on substandard firm
ra          % gross return on bank's asset

% Shocks
ea          % neutral technology shock
eb          % intertemporal preference shock
ei          % investment shock
eg          % government spending shock
em          % monetary policy shock
ep          % price markup shock
ew          % wage markup shock or labor supply shock
sigma       % standard deviation of island-specific shock
sigmaN      % volatility news
theta       % banks continuation probability

% Observable Variables
da dc di dn dw dx lo po ro
;

varexo eea eeb eeg eei eem eep ees eesN eet eew;  


%--------------------------------------------------------------------------
%                               Parameters
%--------------------------------------------------------------------------
parameters  

% Standard Part
alpha       % capital share in production
beta        % discount factor
chi         % elasticity of capital utilization cost
delta       % depreciation rate
gammabar    % gdp trend
gss         % steady state spending output ratio
h           % habit formation in consumption
iotap       % degree of price indexation
iotaw       % degree of wage indexation
lambdap     % steady state price markup
lambdaw     % steady state wage markup
lbar        % log steady state hours
lss         % steady state hours
phip        % Taylor rule inflation coefficient
phix        % Taylor rule output coefficient
phidx       % Taylor rule output growth coefficient
pbar        % inflation trend
rhor        % Taylor rule smoothing coefficient
spp         % investment adjustment costs
v           % inverse frisch elasticity
xip         % calvo price stickiness
xiw         % calvo wage stickiness

% Financial Part
eta         % variance of substandard technology
phiss       % steady state leverage
psi         % mean substandard technology
sigmass     % steady state island-specific volatility
tau         % equity injections in new banks
thetass     % continuation probability of banks

% Shocks
rhoa        % technnology smoothing
rhob        % preference smoothing
rhog        % government spending smoothing                      
rhoi        % investment smoothing
rhom        % monetary policy smoothing
rhop        % price markup smoothing
rhos        % island-specific volatility smoothing
rhosN       % volatility news smoothing
rhot        % equity smoothing
rhow        % wage markup smoothing
thetap      % price markup moving average
thetaw      % wage markup moving average
;              

% Standard Part
alpha = 0.30; 
beta = 1.01;
chi = 5;  
delta = 0.025;
gammabar = 0.50; 
gss = 1.282;
h = 0.7;   
iotap = 0.5;
iotaw = 0.5;
lambdap = 0.15;
lambdaw = 0.15;
lss = 1;
lbar = log(lss);
pbar = 0.50;    % corresponds to pss = 1.005
phip = 1.5;
phix = 0.125;
phidx = 0.125;
rhor = 0.75;
spp = 4;
v = 2; 
xip = 0.66;
xiw = 0.66;

% Financial Part
eta = 1.2691;
phiss = 27.42;  % leverage of security broker-dealers
psi = 0.001;
sigmass = 0.0373;
tau = 0.0207;
thetass = 0.75;

% Shocks
rhoa = 0.5;
rhob = 0.5; 
rhog = 0.5;          
rhoi = 0.5;
rhom = 0;
rhop = 0.5;
rhos = 0.9457;
rhosN = 0.5;
rhot = 0;
rhow = 0.5;
thetap = 0.5;
thetaw = 0.5;


%--------------------------------------------------------------------------
%                                   Model
%--------------------------------------------------------------------------
model(linear); 

% Standard Part - Parameters and Steady State Ratios

#gamma = gammabar/100+1;            % ss growth rate of gdp
#rkss = gamma/beta-(1-delta);       % ss real rental rate of capital
#wss = (1/(1+lambdap)*alpha^alpha*  % ss real wage
(1-alpha)^(1-alpha)*1/(rkss^alpha))
^(1/(1-alpha));              
#kl = wss/rkss*alpha/(1-alpha);     % capital labor ratio
#fl = kl^alpha-rkss*kl-wss;         % fixed cost labor ratio
#yl = kl^alpha-fl;                  % output labor ratio
#il = (1-(1-delta)/gamma)*gamma*kl; % investment labor ratio
#cl = yl*1/gss-il;                  % consumption labor ratio
#ll = cl^-1*(gamma-h*beta)/(gamma-h); % ss lambda x ss labor!!Not a ratio!!
#lambdass = ll/lss;                 % ss marginal utility of income
#kss = kl*lss;                      % ss capital
#fss = fl*lss;                      % ss fixed cost
#yss = yl*lss;                      % ss output
#iss = il*lss;                      % ss investment
#css = cl*lss;                      % ss consumption
#varphi = lss^-v*wss                % parameter to pin down ss hours
*lambdass/(1+lambdaw);                        
#kappa = (1-xip*beta)*(1-xip)       % simplif param in NK Phillips curve
/(xip*(1+iotap*beta));                           
#kappaw = (1-xiw*beta)*(1-xiw)      % simplif param in wage Phillips curve
/(xiw*(1+beta)*(1+v*(1+1/lambdaw)));                           
#mss = beta/gamma;                  % ss stochastic discount factor
#pss = pbar/100+1;                  % ss inflation
#rss = gamma*pss/beta;              % interest rate trend
#rbar = (rss-1)*100;                % steady state interest rate

% Financial Part - Parameters and Steady State Ratios

#ass = kss;                         % ss assets  
#Rbar = rss*1.0025^(1/4);           % interest rate paid by bank on debt
#bss = phiss/(phiss-1)*Rbar;        % ss debt
#nss = kss/phiss;                   % ss net worth
#rass = rkss+1-delta;               % ss return on bank assets
#oss = bss/rass;                    % ss default threshold
#piss = oss-(1-1/phiss)/(mss*rass); % ss value put option standard firm  
#pitss = 1-exp(-psi/2)+piss;        % ss value put option substandard firm
#Fss = normcdf((log(oss)            % ss cdf standard firm
 +sigmass^2/2)/sigmass);                                   
#Ftss = normcdf((log(oss)           % steady state cdf substandard firm
 +(psi+eta*sigmass^2)/2)
 /(eta^(1/2)*sigmass));                                 
#Jss = normcdf((log(oss)            % simplif param put option standard
 -(sigmass^2)/2)/sigmass);                      
#Jtss = normcdf((log(oss)           % simplif param put option substandard
 +(psi-eta*sigmass^2)/2)
 /(eta^(1/2)*sigmass));                         
#muss = (mss*rass*(1-thetass)       % steady sate multiplier bank problem
 *(1-oss+piss))/(1-mss*rass*(oss-piss)
 -thetass*mss*rass*(1-oss+piss));


% Standard Part - Model Equations

% 1) Production function
y = (yss+fss)/yss * (alpha*k + (1-alpha)*l + ea);

% 2) Real rental rate of capital
rk = w + l - k;

% 3) Marginal cost
s = alpha*rk + (1-alpha)*w - ea;

% 4) New Keynesian Phillips curve !! Normalized price markup shock !!
p = beta/(1+iotap*beta)*p(+1) + iotap/(1+iotap*beta)*p(-1) + kappa*s + ep;

% 5) Marginal utility  !! Normalized preference shock !!
lambda = (h*beta*gamma)/((gamma-h*beta)*(gamma-h))*c(+1) 
         - (gamma^2+h^2*beta)/((gamma-h*beta)*(gamma-h))*c 
         + (h*gamma)/((gamma-h*beta)*(gamma-h))*c(-1) + eb;

% 6) Bonds Euler
lambda = r + lambda(+1) - p(+1);

% 7) Participation constraint or new capital Euler
lambda = ra(+1) + lambda(+1) + oss/(oss-piss)*o(+1) 
         - piss/(oss-piss)*pi(+1) - 1/(phiss-1)*phi;

% 8) Return on bank assets
ra = rkss/(rkss+1-delta)*rk + (1-delta)/(rkss+1-delta)*q - q(-1);

% 9) Investment Euler
0 = q + ei - gamma^2*spp*(i - i(-1)) + beta*gamma^2*spp*(i(+1) - i); 

% 10) Capital utilization
rk = chi*u;

% 11) Capital input
k = u + kbar(-1);

% 12) Capital accumulation
kbar = (1-delta)/gamma*kbar(-1) + (1-(1-delta)/gamma)*(ei + i);

% 13) Wage Phillips curve  !! Normalized wage markup shock !!
w = 1/(1+beta)*w(-1) + beta/(1+beta)*w(+1) 
    - kappaw*(w - v*l - eb + lambda) + iotaw/(1+beta)*p(-1) 
    - (1+beta*iotaw)/(1+beta)*p + beta/(1+beta)*p(+1) + ew;

% 14) Taylor rule !! modified wrt to JPT !!
% r = rhor*r(-1) + (1-rhor)*(phip*p + phix*x) + phidx*(x - x(-1)) + em;
r = rhor*r(-1) + (1-rhor)*(phip*p + phix*y) + phidx*(y - y(-1)) + em;

% 15) Definition of GDP
x = y - rkss*kss/yss*u;

% 16) Aggregate resource constraint
1/gss*y = css/yss*c + iss/yss*i + rkss*kss/yss*u + 1/gss*eg;


% Financial Part - Model Equations

% 17) Incentive compatibility constraint
0 = piss*pi(+1) - pitss*pit(+1);

% 18) Capital = net worth * leverage
k = phi + n;

% 19) Threshold level of island-specific shock
o = b(-1) - ra;

% 20) Net worth
(nss/(nss-tau*kss*(1-thetass+thetass*Fss)))*n = 
(nss/(nss-tau*kss*(1-thetass+thetass*Fss)))*k(-1) 
+ ((tau*kss*thetass*Fss)/(nss-tau*kss*(1-thetass+thetass*Fss)))*F + ra 
+ ((nss-tau*kss)/(nss-tau*kss*(1-thetass+thetass*Fss)))*theta
+ (piss/(1-oss+piss))*pi - (oss/(1-oss+piss))*o;

% 21) Lagrange multiplier on bank's problem
mu = (1/(1-beta*rass*(oss-piss)))*c - (1/(1-beta*rass*(oss-piss)))*c(+1)
     + (1/(1-beta*rass*(oss-piss)))*ra(+1) - (oss/(1-oss+piss)-(beta*css
     *rass*oss)/(css-beta*css*rass*(oss-piss)))*o(+1) + (piss/(1-oss+piss)
     -(beta*css*rass*piss)/(css-beta*css*rass*(oss-piss)))*pi(+1) 
     +(thetass*muss/(thetass*muss+1-thetass))*mu(+1);

% 22) Put option value, standard firm
pi = (oss*Fss/(oss*Fss-Jss))*(o + F) - Jss/(oss*Fss-Jss)*(1/Jss * 
     ( 1/(2*sqrt(3.1415926)) * exp(-((log(oss)-sigmass^2/2)/sigmass)^2/2)
     *( 1/sigmass*o - ((sigmass^2+2*log(oss))/(2*sigmass))*sigma(-1) ) ));

% 23) Put option value, substandard firm
pit = (oss*Ftss/(oss*Ftss-Jtss))*(o + Ft) - Jtss/(oss*Ftss-Jtss)*(1/Jtss 
      * ( 1/(2*sqrt(3.1415926)) * exp((-((log(oss)+(psi-eta*sigmass^2)/2)
     /(sqrt(eta)*sigmass))^2)/2) * ( 1/(sqrt(eta)*sigmass)*o 
     - (eta*sigmass^2+2*log(oss)+psi)/(2*sqrt(eta)*sigmass)*sigma(-1) ) ));

% 24) Cdf of island-specific shock, standard firm
F = 1/Fss * ( 1/(2*sqrt(3.1415926)) 
    * exp(-((log(oss)+sigmass^2/2)/sigmass)^2/2)
    * ( 1/sigmass*o + ((sigmass^2-2*log(oss))/(2*sigmass))*sigma(-1) ) );

% 25) Cdf of island-specific shock, substandard firm
Ft = 1/Ftss * ( 1/(2*sqrt(3.1415926)) 
     * exp((-((log(oss)+(psi+eta*sigmass^2)/2)/(sqrt(eta)*sigmass))^2)/2) 
     * ( 1/(sqrt(eta)*sigmass)*o 
     + (eta*sigmass^2-2*log(oss)-psi)/(2*sqrt(eta)*sigmass)*sigma(-1) ) );

% 26) Bank assets
a = k;

% 27) Bank debt
B = ass/(ass-nss)*a - nss/(ass-nss)*n;

% 28) Face value of debt
Bbar = b + a;

% 29) Bank debt haircut
hc = - phi;


% Shocks

% 30) Technology
ea = rhoa*ea(-1) + eea;

% 31) Preference
eb = rhob*eb(-1) + eeb;

% 32) Government spending
eg = rhog*eg(-1) + eeg; 

% 33) Investment
ei = rhoi*ei(-1) + eei;

% 34) Monetary policy
em = rhom*em(-1) + eem; 

% 35) Price markup
ep = rhop*ep(-1) + eep - thetap*eep(-1);

% 36) Wage markup
ew = rhow*ew(-1) + eew - thetaw*eew(-1);

% 37) Standard deviation of island-specific shock
sigma = rhos*sigma(-1) + sigmaN(-4) + ees;

% 39) Volatility news shock
sigmaN = rhosN*sigmaN(-1) + eesN;

% 38) Equity or net worth
theta = rhot*theta(-1) + eet;


% Measurement Equations

da = a - a(-1) + gammabar;
dc = c - c(-1) + gammabar;
di = i - i(-1) + gammabar;
dn = n - n(-1) + gammabar;
dw = w - w(-1) + gammabar;
dx = x - x(-1) + gammabar;
lo = l + lbar;
po = p + pbar;
ro = r + rbar;

end;


%--------------------------------------------------------------------------
%                                 Shocks
%--------------------------------------------------------------------------
shocks;
var eea; stderr 1;
var eeb; stderr 1;
var eeg; stderr 1;
var eei; stderr 1;
var eem; stderr 1;
var eep; stderr 1;
var ees; stderr 1;
var eesN; stderr 1;
var eet; stderr 1;
var eew; stderr 1;
end;


%--------------------------------------------------------------------------
%                                 IRFs
%--------------------------------------------------------------------------
resid;
steady;
check;
% stoch_simul(irf=40) y l i phi a r rk ra c;
% model_diagnostics(M_,options_,oo_);


%--------------------------------------------------------------------------
%                          Bayesian Estimation
%--------------------------------------------------------------------------
%@#if 0
varobs dx dc di dw lo po ro da dn sigma;

estimated_params;

% Standard Parameters
chi,5,gamma_pdf,5,1;
gammabar,0.5,normal_pdf,0.5,0.1; 
h,0.7,beta_pdf,0.7,0.1; 
iotap,0.5,beta_pdf,0.5,0.15;
iotaw,0.5,beta_pdf,0.5,0.15;
lambdap,0.15,normal_pdf,0.15,0.05;
lambdaw,0.15,normal_pdf,0.15,0.05;
lbar,0,normal_pdf,0.00,2.00;
pbar,0.5,gamma_pdf,0.62,0.1;
phip,1.5,normal_pdf,1.5,0.1;        
phix,0.125,normal_pdf,0.125,0.05;
phidx,0.125,normal_pdf,0.125,0.05;
rhor,0.70,beta_pdf,0.70,0.2;
spp,4,gamma_pdf,4,1;
v,2,gamma_pdf,2,0.75; 
xip,0.66,beta_pdf,0.66,0.1;         
xiw,0.5,beta_pdf,0.66,0.1;          

% Financial Parameters
stderr sigma, 0.1, inv_gamma_pdf, 0.1, 1;  % measurement error
% eta = 1.2691;     !! not estimated? !!
% psi = 0.001;      !! not estimated? !!
% sigmass = 0.0373; !! not estimated? !!
% tau,0.0207,beta_pdf,0.02,0.1,0.01,1;
% thetass,0.75,beta_pdf,0.75,0.1;

% Shocks
rhoa,0.5,beta_pdf,0.5,0.05;         
rhob,0.5,beta_pdf,0.5,0.05;         
rhog,0.5,beta_pdf,0.5,0.05;         
rhoi,0.5,beta_pdf,0.5,0.05;         
rhop,0.5,beta_pdf,0.5,0.05;         
rhos,0.5,beta_pdf,0.5,0.05;         
rhosN,0.5,beta_pdf,0.5,0.05;     
rhow,0.5,beta_pdf,0.5,0.05;       
thetap,0.5,beta_pdf,0.5,0.2;        
thetaw,0.5,beta_pdf,0.5,0.2;        
stderr eea,0.1,inv_gamma_pdf,1,1;  
stderr eeb,0.1,inv_gamma_pdf,1,1; 
stderr eeg,0.1,inv_gamma_pdf,0.1,2;  
stderr eei,0.1,inv_gamma_pdf,0.1,2;
stderr eem,0.1,inv_gamma_pdf,0.1,2;
stderr eep,0.1,inv_gamma_pdf,0.1,2;
stderr ees,0.1,inv_gamma_pdf,0.1,2;
stderr eesN,0.1,inv_gamma_pdf,0.1,2;
stderr eew,0.1,inv_gamma_pdf,0.1,2;
stderr eet,0.1,inv_gamma_pdf,0.1,2;

end;

identification(advanced=1,parameter_set=prior_mean);

estimation(datafile='PCdata.mat',plot_priors=0,mh_replic=200000,
mh_nblocks=2,mh_drop=0.4,mode_compute=9,mode_check,prior_trunc=0,
bayesian_irf,irf=20,moments_varendo,conditional_variance_decomposition
=[1,4,8,16,32]) dx dc di dw lo po ro da dn sigma;

%@#endif

